"""
Django settings for WebTVPortals project.

Generated by 'django-admin startproject' using Django 2.0.8.

For more information on this file, see
https://docs.djangoproject.com/en/2.0/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/2.0/ref/settings/
"""

import os

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
PROJECT_APP_PATH = os.path.dirname(os.path.abspath(__file__))
PROJECT_APP = os.path.basename(PROJECT_APP_PATH)
PROJECT_ROOT = BASE_DIR = os.path.dirname(PROJECT_APP_PATH)

# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/2.0/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = '_@+v(@4dd9%8n-viza=@o57hkg7pn96*7=uj&$w)i#qijnovoa'

SECURITY_KEY = ""
DATA_UPLOAD_MAX_NUMBER_FIELDS = 10240
PREFIX_SERIAL_NUMBER = "1601WT"

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

ALLOWED_HOSTS = ['*']
AUTH_USER_MODEL = "distributor.User"
# Application definition

INSTALLED_APPS = [
    'WebTVPortals.apps.CustomAdminConfig',
    'corsheaders',
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'apkparser',
    'appstore',
    'portal',
    'device',
    'distributor',
    'import_export',
    'ini_settings',
    'nested_inline',
    'ordered_model',
    'rest_framework',
]

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
]

CORS_ALLOW_METHODS = (
    'DELETE',
    'GET',
    'OPTIONS',
    'PATCH',
    'POST',
    'PUT',
    'VIEW',
)
CORS_ALLOW_HEADERS = (
    'XMLHttpRequest',
    'X_FILENAME',
    'accept-encoding',
    'authorization',
    'content-type',
    'dnt',
    'origin',
    'user-agent',
    'x-csrftoken',
    'x-requested-with',
    'Pragma',
    'sn',
    'name',
    'password',
)

ROOT_URLCONF = 'WebTVPortals.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [os.path.join(BASE_DIR, 'templates')],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

WSGI_APPLICATION = 'WebTVPortals.wsgi.application'


# Database
# https://docs.djangoproject.com/en/2.0/ref/settings/#databases

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.mysql', 
        'NAME': 'webtv', 
        'USER': 'root',
        'PASSWORD': '',
        'HOST': '127.0.0.1',         
        'PORT': '3306',
        'OPTIONS':{
            "init_command": "SET foreign_key_checks = 0;",
        },
    }
}


# Password validation
# https://docs.djangoproject.com/en/2.0/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',
    },
]


# Internationalization
# https://docs.djangoproject.com/en/2.0/topics/i18n/

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/2.0/howto/static-files/

STATIC_URL = '/static/'
STATICFILES_DIRS = (
    (os.path.join(BASE_DIR,'static')),
)

import sys
from ini_settings.load import load_ini
from ini_settings.settings_types import (DictItemSetting, StringSetting,
                                           BoolSetting, IntSetting)

CONFIG_FILE_MAPPING = {
    ("database", "user"): DictItemSetting(("DATABASES", "default", "USER")),
    ("database", "password"): DictItemSetting(("DATABASES", "default", "PASSWORD")),
    ("database", "engine"): DictItemSetting(("DATABASES", "default", "ENGINE")),
    ("database", "name"): DictItemSetting(("DATABASES", "default", "NAME")),
    ("database", "host"): DictItemSetting(("DATABASES", "default", "HOST")),
    ("database", "port"): DictItemSetting(("DATABASES", "default", "PORT")),
    ("media", "url"): StringSetting("MEDIA_URL"),
    ("media", "root"): StringSetting("MEDIA_ROOT"),
    ("static", "url"): StringSetting("STATIC_URL"),
    ("static", "root"): StringSetting("STATIC_ROOT"),
    ("site", "language"): StringSetting("LANGUAGE_CODE"),
    ("security", "encryptkey"): StringSetting("SECURITY_KEY"),
}


load_ini(ini_file=BASE_DIR+"/config.ini",
                  settings_module=sys.modules[__name__],
                  config_mapping=CONFIG_FILE_MAPPING)