# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.contrib import admin
from django import forms
from device.models import device,group
from portal.models import portal
from django.contrib.admin.widgets import FilteredSelectMultiple

# Register your models here.
class GroupAdminForm(forms.ModelForm):
    class Meta:
        model = group
        fields = "__all__" 

    portals = forms.ModelMultipleChoiceField(
        queryset=portal.objects.filter(enable=True).order_by('order').all(),
        required=False,
        widget=FilteredSelectMultiple(
            verbose_name='portals',
            is_stacked=False,
            attrs={'size':'20'}
        )
    )

    def __init__(self, *args, **kwargs):
        super(GroupAdminForm, self).__init__(*args, **kwargs)
        self.fields['portals'].queryset = portal.objects.filter(enable=True).order_by('order').all()
        if self.instance.pk:
            self.fields['portals'].initial = self.instance.portals.all()

    def save(self, commit=True):
        project = super(GroupAdminForm, self).save(commit=False)  
        if commit:
            project.save()

        if project.pk:
            project.portals.set(self.cleaned_data['portals'])
            self.save_m2m()

        return project
    
class ExtendGroup(admin.ModelAdmin): 
    list_display = ('name','id',)
    fields = ('name','portals')
    search_fields = ['name',]
    list_per_page = 20
    form = GroupAdminForm
    
    def get_queryset(self, request):
        qs = group.objects.filter(dealer_id=request.user.id).all()
        return qs
    
    def save_model(self, request, obj, form, change):
        form.save()
        obj.dealer_id = request.user.id
        obj.save()


class ExtendRecord(admin.ModelAdmin): 
    list_display = ('device_id','username','mygroup','code','dealer','hw_version','modelname','vendor','sn','mac','ip','modifyTime','enable')
    list_editable = ('enable', )
    list_filter=['mygroup',]
    search_fields = ['device_id','username','modelname','vendor','ip','sn','mac',]
    list_per_page = 20
    change_form_template = None
    
    def get_form(self, request, obj=None, **kwargs):
        form = super(ExtendRecord, self).get_form(request, obj, **kwargs)

        form.base_fields['mygroup'].queryset = group.objects.filter(dealer_id=request.user.id).all()

        return form
    
    def has_add_permission(self, request):
        return False
    
    def has_delete_permission(self, request, obj=None):
        return False
    
    def has_change_permission(self, request, obj=None):
        if request.user.is_superuser:
            self.list_filter=['mygroup',]
        else:
            self.list_filter=[]
        if obj:
            self.readonly_fields=('device_id','code','dealer','hw_version','modelname','vendor','sn','mac','ip','modifyTime','enable')
            self.exclude = ('dealer',)
            return True
        return True
    
    def get_queryset(self, request):
        if request.user.is_superuser:
            qs = super(ExtendRecord, self).get_queryset(request)
            return qs
        else:
            qs = device.objects.filter(dealer_id=request.user.id).all()
            return qs
    
    def get_mygroups(self, request):
        qs = group.objects.filter(dealer_id=request.user.id).all()
        return qs
    
admin.site.register(device , ExtendRecord)
admin.site.register(group , ExtendGroup)
