# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import json,re
from rest_framework.decorators import api_view
from rest_framework.response import Response
from rest_framework.views import APIView
from rest_framework import permissions
from rest_framework import authentication
from rest_framework import status
from django.utils import timezone
from WebTVPortals import settings
from WebTVPortals import message
from device.models import device,activeCode
from portal.models import portal,portalSerializer

def getMacFromSn(sn):
    pattern = re.compile(r"\d{4,6}")
    result = pattern.findall(sn)
    ym = int(result[0])
    serial = int(result[1])
    y = int((ym/100)-14)
    m = ym%100
    serial = m*40000+serial
    mac = "%x"%(y * 0x7ffff + serial + 0x800000)
    return "00:1A:79:%s:%s:%s"%(mac[0:2],mac[2:4],mac[4:6])
    
    
def getAllocatedHardwareInfo():
    total = device.objects.count()+1
    sn = settings.PREFIX_SERIAL_NUMBER+str(total).zfill(6)
    return sn, getMacFromSn(sn)

# Create your views here.
@api_view(['GET', 'POST', ])        
def registeryView(request):
    responseMsg={}
    code = request.GET.get('code')
    ipAddr = request.META.get('REMOTE_ADDR','')
    
    if not code or not request.body:
        responseMsg['status'] = -3
        print ("code does not appended,ip=%s"%ipAddr)
        return message.json_response(responseMsg, settings.SECURITY_KEY, True)
    else:
        cryptor = message.AESCipher(settings.SECURITY_KEY)
        payload = cryptor.decrypt(request.body)
                        
        try:
            payload = json.loads(payload)
            
            deviceid = "" if 'deviceid' not in payload else payload['deviceid']
            vendor = "" if 'vendor' not in payload else payload['vendor']
            model = "" if 'model' not in payload else payload['model']
            
            hDevice = device.objects.filter(code__code=code).first()
            if hDevice: 
                if deviceid!=hDevice.device_id:
                    hDevice.device_id=deviceid
                    hDevice.save()
                responseMsg['status'] = 0
                responseMsg['sn'] = hDevice.sn
                responseMsg['mac'] = hDevice.mac
                print ("code=%s used, device match,ip=%s,device=%s,vendor=%s,model=%s"%(code,ipAddr,deviceid,vendor,model))
                return message.json_response(responseMsg, settings.SECURITY_KEY, True)
            else:
                active = activeCode.objects.filter(code=code).first()
                if active:
                    hDevice = device(code=active)
                    hDevice.device_id=deviceid
                    hDevice.dealer_id = active.dealer_id
                    hDevice.vendor=vendor
                    hDevice.modelname=model
                    hDevice.ip=ipAddr
                    sn, mac = getAllocatedHardwareInfo()
                    hDevice.sn = sn
                    hDevice.mac = mac
                    hDevice.save()
                    
                    active.actived = True
                    active.activeTime = timezone.now()
                    active.save()
                    responseMsg['status'] = 0                
                    responseMsg['sn'] = hDevice.sn
                    responseMsg['mac'] = hDevice.mac
                    
                    print ("code=%s actived,ip=%s,device=%s,vendor=%s,model=%s"%(code,ipAddr,deviceid,vendor,model))
                    return message.json_response(responseMsg, settings.SECURITY_KEY, True)
                else:
                    responseMsg['status'] = -1
                    print ("code=%s failed,ip=%s,device=%s,vendor=%s,model=%s"%(code,ipAddr,deviceid,vendor,model))
                    return message.json_response(responseMsg, settings.SECURITY_KEY, True)
        except Exception as e:
            responseMsg['status'] = -3
            responseMsg['sn'] = ""
            responseMsg['enable'] = False
            print ("exception=%s,ip=%s"%(str(e),ipAddr))
            return message.json_response(responseMsg, settings.SECURITY_KEY)
        
@api_view(['GET', 'POST', ])        
def restoreView(request):
    responseMsg={}
    ipAddr = request.META.get('REMOTE_ADDR','')
    
    if not request.body:
        responseMsg['status'] = -3
        print ("code does not appended,ip=%s"%ipAddr)
        return message.json_response(responseMsg, settings.SECURITY_KEY, True)
    else:
        cryptor = message.AESCipher(settings.SECURITY_KEY)
        payload = cryptor.decrypt(request.body)
                        
        try:
            payload = json.loads(payload)
            
            deviceid = "" if 'deviceid' not in payload else payload['deviceid']
            vendor = "" if 'vendor' not in payload else payload['vendor']
            model = "" if 'model' not in payload else payload['model']
            
            hDevice = device.objects.filter(device_id=deviceid).first()
            if hDevice: 
                responseMsg['status'] = 0
                responseMsg['sn'] = hDevice.sn
                responseMsg['mac'] = hDevice.mac
                print ("device match,ip=%s,device=%s,vendor=%s,model=%s"%(ipAddr,deviceid,vendor,model))
                return message.json_response(responseMsg, settings.SECURITY_KEY, True)
            else:
                responseMsg['status'] = -1
                print ("ip=%s,device=%s,vendor=%s,model=%s"%(ipAddr,deviceid,vendor,model))
                return message.json_response(responseMsg, settings.SECURITY_KEY, True)

        except Exception as e:
            responseMsg['status'] = -2
            responseMsg['sn'] = ""
            responseMsg['enable'] = False
            print ("exception=%s,ip=%s"%(str(e),ipAddr))
            return message.json_response(responseMsg, settings.SECURITY_KEY)
    
@api_view(['GET', 'POST', ])        
def getHardwareView(request):
    sn = request.GET.get('sn')
    mac = getMacFromSn(sn)
    responseMsg={}
    responseMsg['sn']=sn
    responseMsg['mac']=mac  
    return message.json_response(responseMsg, settings.SECURITY_KEY, True)

def isValidSn(sn):  
    if re.match(r"[0-9a-zA-Z]{8,12}", sn): return True  
    print ("not valid sn")
    return False 

def isValidMac(mac):  
    if re.match(r"^\s*([0-9a-fA-F]{2,2}:){5,5}[0-9a-fA-F]{2,2}\s*$", mac): return True
    print ("not valid mac")
    return False 

class DeviceAuthentication(authentication.BaseAuthentication):
    def authenticate(self, request):
        serialnumber = request.META.get('HTTP_SN')
        mac = request.META.get('HTTP_MAC')

        if (serialnumber is None) or (mac is None):
            return (None, None)
        else:
            if (isValidMac(mac)==False) or (isValidSn(serialnumber)==False):
                return (None, None)
            try:
                client = device.objects.filter(sn=serialnumber).first()
                if client:
                    return (client, None)
            except device.DoesNotExist:
                print ("No such user")
                return (None, None)
        return (None, None)
    
class DevicePortalViewSet(APIView):
    authentication_classes = (DeviceAuthentication, )
    
    def feedback(self, request, format=None):
        user = request.user
        if user is not None:
            if user.mygroup:
                portals = user.mygroup.portals.all()
            else:
                portals = portal.objects.filter(enable=True).order_by('order').all()
        else:
            portals = portal.objects.filter(enable=True).order_by('order').all()
        
        serializer = portalSerializer(data=portals, many=True)
        serializer.is_valid()
        responseMsg={}
        responseMsg['status'] = 0
        responseMsg['data'] = serializer.data
        return message.json_response(responseMsg, settings.SECURITY_KEY, True)
        
    def get(self, request, format=None): 
        return self.feedback(request, format)
    
    def post(self, request, format=None):
        return self.feedback(request, format)