import random
from django import forms
from django.contrib import admin
from django.contrib.auth.admin import UserAdmin

from distributor.models import User,activeCode,activeCodeSellInfo
from django.contrib.admin.widgets import FilteredSelectMultiple
from django.db.models import Q
from django.contrib.auth.forms import UserChangeForm,UserCreationForm
from import_export.admin import ExportMixin,ImportExportMixin
from import_export.resources import ModelResource
# Register your models here.

class codeResource(ModelResource):
    class Meta:
        model = activeCode
        fields=['code','cretateTime']
    
class ActiveCodeAdmin(ExportMixin, admin.ModelAdmin):
    list_display = ('code','dealer','cretateTime','activeTime','actived','enable')    
    list_filter=['actived',]
    search_fields = ['code',]
    resource_class = codeResource
    list_per_page = 20
    change_form_template = None
    
    def get_queryset(self, request):
        if request.user.is_superuser:
            self.readonly_fields=('code','actived','enable')
            self.fields = ('quantity',)
            self.exclude = ('dealer','cretateTime','activeTime')
            qs = super(ActiveCodeAdmin, self).get_queryset(request)
            return qs
        else:
            self.readonly_fields=('code','actived','enable')
            self.fields = ('code',)
            self.list_editable=()
            self.exclude = ('dealer','quantity','cretateTime','activeTime')
            qs = activeCode.objects.filter(dealer_id=request.user.id).all()
            return qs
    
    def has_add_permission(self, request):
        if request.user.is_superuser:
            return True
        else:
            return False
    
    def has_delete_permission(self, request, obj=None):
        if request.user.is_superuser:
            return True
        else:
            return False
        
    def has_change_permission(self, request, obj=None):
        if obj:return False
        return True
        
        
    def save_model(self, request, obj, form, change):
        def random_creator(count):
            allCodes = activeCode.objects.values("code").all()            
            index=0
            createdCodes=[]
            while index<count:
                pCode = random.randint(10000000,99999999)
                if pCode not in allCodes:
                    createdCodes.append(pCode)
                    index = index+1
            return createdCodes

        quantity = obj.quantity
        
        allCodes = []
        if change is False:
            if quantity>1:
                allCodes = random_creator(quantity-1)
                code_list_to_insert = list()
                for x in allCodes:
                    code_list_to_insert.append(activeCode(code=x,dealer_id=request.user.id))
                activeCode.objects.bulk_create(code_list_to_insert)
                
            codes = random_creator(1)
            obj.code=codes[0]
            obj.dealer_id=request.user.id
            obj.save()
        else:
            obj.save()
        form.save()


           
class dealerCreateForm(UserCreationForm):        
    class Meta(UserChangeForm.Meta):
        model = User
        fields = ('username', 'email','first_name','last_name','password1', 'password2')

    activeCodes = forms.ModelMultipleChoiceField(
        queryset=None,
        required=False,
        widget=FilteredSelectMultiple(
            verbose_name='Active code',
            is_stacked=False,
            attrs={'size':'20'}
        )
    )
    
    def __init__(self, *args, **kwargs):
        super(dealerCreateForm, self).__init__(*args, **kwargs)
        self.fields['activeCodes'].queryset = activeCode.objects.filter(enable=True).filter(actived=False).all().filter(dealer_id=self.current_user.id)        
        if self.instance.pk:
            self.fields['activeCodes'].initial = self.instance.codes.all()
        

    def save(self, commit=True):
        project = super(dealerCreateForm, self).save(commit=False)
        if commit:
            print ("data=%s"%(self.cleaned_data['activeCodes']))
            #prepaidSellInfo.objects.create(prepay=tag, codeOwner=project)
            #project.save()

        if project.pk:
            project.codes = self.cleaned_data['activeCodes']
            self.save_m2m()

        return project
    
class dealerChangeForm(UserChangeForm):    
    class Meta(UserChangeForm.Meta):
        model = User
        fields = ('username','email','first_name','last_name','last_login','password')

    activeCodes = forms.ModelMultipleChoiceField(
        queryset=None,
        required=False,
        widget=FilteredSelectMultiple(
            verbose_name='Active code',
            is_stacked=False,
            attrs={'size':'20'}
        )
    )
    
    def __init__(self, *args, **kwargs):
        super(dealerChangeForm, self).__init__(*args, **kwargs)
        self.fields['activeCodes'].queryset = activeCode.objects.filter(Q(dealer_id=self.current_user.id)|Q(dealer_id=self.instance.id)).filter(enable=True).filter(actived=False).all()
            
        if self.instance.pk:
            self.fields['activeCodes'].initial = self.instance.codes.all()


    def save(self, commit=True):
        project = super(dealerChangeForm, self).save(commit=False)  

        if commit:
            project.save()

        if project.pk:
            project.codes.clear()
            codes = self.cleaned_data['activeCodes']
                        
            tag_list_to_insert = list()
            allocated_code_list = []
            for c in codes:
                tag_list_to_insert.append(activeCodeSellInfo(code=c, codeOwner=project, sold=False))
                allocated_code_list.append(c.id)
                c.dealer_id = self.instance.id
                c.save()
                           
            refunds = activeCode.objects.filter(Q(dealer_id=self.instance.id)).filter(actived=False).exclude(id__in=allocated_code_list).all()
            for c in refunds:
                c.dealer_id = self.current_user.id
                c.save()
            
            activeCodeSellInfo.objects.bulk_create(tag_list_to_insert)
            
            self.save_m2m()

        return project
    
class ImportResource(ModelResource):
    class Meta:
        model = User
        fields=['id','username','email','first_name','last_name','CreatedBy','last_login','password']

class CustomUserAdmin(UserAdmin):
    #inlines = [TariffPlanInline,]
    list_display = ('username','email','first_name','last_name','CreatedBy','last_login')
    fieldsets = (
        (None, {'fields': ('username', 'password',)}),
        ('Personal info', {'fields': ('first_name', 'last_name', 'email')}),
        ('ActiveCode info', {'fields': ('activeCodes',)}),
    )
    add_fieldsets = (
        (None, {
            'classes': ('wide',),
            'fields': ('username', 'email','first_name','last_name','password1', 'password2',),
        }),
    )
    search_fields = ['username','email',]
    list_filter = ()
    list_per_page = 20
    
    def get_form(self, request, obj=None, **kwargs):
        if obj is None:
            form = dealerCreateForm
        else:
            form = dealerChangeForm
        form.current_user = request.user
        form.sellto = obj
        return form
    
    def CreatedBy(self,obj):
        if obj.is_superuser:
            return "SuperUser"
        return obj.owner
    
    def get_queryset(self, request):
        qs = super(CustomUserAdmin, self).get_queryset(request)
        if request.user.is_superuser:
            return qs
        allusers = User.objects.values("id").filter(owner=request.user.username).all()
        return qs.filter(id__in=allusers)
    
    def save_model(self, request, obj, form, change):
        form.save()
        
        obj.owner = request.user.username
        obj.is_staff = True
        obj.save()

admin.site.register(User, CustomUserAdmin)
admin.site.register(activeCode, ActiveCodeAdmin)