'''
@author: ShaoYufeng
'''
import re,json
from datetime import datetime,timedelta
from rest_framework import serializers,status
from rest_framework.response import Response
from rest_framework import authentication
from rest_framework import exceptions
from rest_framework.views import APIView
from rest_framework import permissions
from rest_framework.authentication import SessionAuthentication, BasicAuthentication
from django.db.models import Q
from django.core.cache import cache
from django.http import HttpResponse
from django.utils import timezone
from django.utils.translation import gettext_lazy as _
from device.models import device,group
from distributor.models import activeCode
from pyecharts.charts import Bar,Pie
from pyecharts import options as opts

# Create your views here.
def response_as_json(data):
    json_str = json.dumps(data)
    response = HttpResponse(
        json_str,
        content_type="application/json",
    )
    response["Access-Control-Allow-Origin"] = "*"
    return response


def json_response(data, code=200):
    data = {
        "code": code,
        "msg": "success",
        "data": data,
    }
    return response_as_json(data)        
        
class DeviceBriefViewSet(APIView):
    authentication_classes = (SessionAuthentication, BasicAuthentication)
    permission_classes = (permissions.IsAuthenticated,)
    
    def get(self, request, format=None):
        user = request.user
        if user is not None:
            time_arry = []
            actvieDevices = []
            groups = group.objects.filter(dealer_id=user.id).all()
            for g in groups:
                time_arry.append(g.name)
                actvieDevices.append(device.objects.filter(Q(dealer_id=user.id),Q(mygroup_id = g.id)).count())
            time_arry.append(str(_('ALL')))
            actvieDevices.append(device.objects.filter(Q(dealer_id=user.id)).count())            
            
            bar = (
                Bar().add_xaxis(time_arry)
                    .add_yaxis(str(_('Actived Devices')), actvieDevices, gap="0%")
                    .set_global_opts(title_opts=opts.TitleOpts(title=_('Actived Devices')),
                                     yaxis_opts=opts.AxisOpts(name=str(_("Actived Devices"))),
                                     xaxis_opts=opts.AxisOpts(name=str(_("Group"))),
                                     )
                    .dump_options_with_quotes()
            )
            return json_response(json.loads(bar))
        else:
            return Response('Not Authorized', status=status.HTTP_401_UNAUTHORIZED)
    
class ActiveCodeBriefViewSet(APIView):
    authentication_classes = (SessionAuthentication, BasicAuthentication)
    permission_classes = (permissions.IsAuthenticated,)
    
    def get(self, request, format=None):
        user = request.user
        if user is not None:
            pie_data_array = []
            used_count = activeCode.objects.filter(Q(dealer_id=user.id),Q(actived=True)).count()
            unused_count = activeCode.objects.filter(Q(dealer_id=user.id),Q(actived=False)).count()
            pie_data_array.append((_("Actived"), used_count))
            pie_data_array.append((_("Not Used"), unused_count))
    
            pie = (
                Pie()
                    .add("", pie_data_array,
                         radius=["0%", "60%"],
                         center=["40%", "50%"],)
                    .set_global_opts(
                        legend_opts=opts.LegendOpts(
                            type_="scroll", pos_left="80%", orient="vertical"),
                    )
                    .set_series_opts(label_opts=opts.LabelOpts(formatter="{b}: {c}"))
                    .dump_options_with_quotes()
            )
            return json_response(json.loads(pie))
        else:
            return Response('Not Authorized', status=status.HTTP_401_UNAUTHORIZED)